
from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import mail_admins, mail_managers
from django.views.generic import FormView
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from .conf import CONTACT_ADMINS_ONLY
from .forms import ContactFormBase


class FeedbackBaseView(FormView):
    """
    Form view that sends email when form is valid. You can subclass it
    and define your own form_class, success_message, success_url and template_name.

    Attributes:
        cls.form_class: Django form class for use to create feedback message. The form must be subclassed from
        'ContactFormBase' at this app.
        cls.success_message (str): Django form class for use to create feedback message.
        cls.success_url (str): URl to redirect user when feedback is sent successfully.
        cls.template_name (str): Name of template file for this view.

    """
    form_class = ContactFormBase
    success_message = _("Message is sent successfully")
    success_url = 'feedback-view'
    template_name = "feedback_form/feedback.html"

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()

        kwargs.update({'request_meta': self.request.META, })

        return kwargs

    def form_valid(self, form):
        mail_data = form.prepare_mail()
        self.send_mail(mail_data)
        return super().form_valid(form)

    def send_mail(self, data: dict):
        result, counts = self._check_receivers_list()

        if result:
            if CONTACT_ADMINS_ONLY:
                if counts[0] > 0:
                    mail_admins(data['subject'], data['message'])
                else:
                    self._raise_config_exception(counts)
            else:
                mail_managers(data['subject'], data['message'])
            messages.success(self.request, self.get_success_message())
        else:
            self._raise_config_exception(counts)

    def get_success_message(self) -> str:
        return self.success_message

    def get_success_url(self):
        return reverse(self.success_url)

    def _check_receivers_list(self) -> (bool, (int, int, )):
        """ Check settings ADMINS and MANAGERS is non empty """
        admin = len(settings.ADMINS)
        managers = len(settings.MANAGERS)

        result = True if admin + managers > 0 else False
        return result, (admin, managers, ),

    def _raise_config_exception(self, counts: tuple):
        """ Raise ImproperlyConfigured with messages about non configured ADMINS or MANAGERS lists """
        r_substr = '__receivers__'
        msg = f"You must add {r_substr} list variable in 'settings' module of your project."
        if CONTACT_ADMINS_ONLY:
            if counts[0] == 0:
                msg = msg.replace(r_substr, 'ADMINS')
        else:
            if counts[0] + counts[1] == 0:
                msg = msg.replace(r_substr, 'ADMINS or MANAGERS')

        raise ImproperlyConfigured(msg)


class FeedbackView(FeedbackBaseView):
    pass




from django import forms
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _
from .conf import CONTACT_SEND_META_INFO


class ContactFormBase(forms.Form):
    name = forms.CharField(label=_("Your name"), max_length=64)
    subject = forms.CharField(label=_("Subject"), max_length=129)
    email = forms.EmailField(label=_("Your email address"), max_length=200)
    body = forms.CharField(label=_("Your message"),
                           widget=forms.Textarea(attrs={'cols': '30', 'rows': '5'}))

    # Non field members
    subject_template = 'feedback_form/email_subject.txt'
    message_template = 'feedback_form/email_template.txt'
    request_meta: dict = None  # request.META

    def __init__(self, request_meta, *args, **kwargs):
        self.request_meta = request_meta
        super().__init__(*args, **kwargs)

    def prepare_mail(self) -> dict:
        mail_dict = {
            'subject': self.get_subject(),
            'message': self.get_message(),
        }
        return mail_dict

    def get_subject(self) -> str:
        return render_to_string(self.subject_template, {'subject': self.cleaned_data['subject'], })

    def get_request_meta(self) -> dict:
        meta = {'IP': self.request_meta['REMOTE_ADDR'], 'USER_AGENT': self.request_meta['HTTP_USER_AGENT']}
        return meta

    def get_message(self) -> str:
        message_context = {
            'name': self.cleaned_data['name'],
            'email': self.cleaned_data['email'],
            'subject': self.get_subject().strip(),
            'body': self.cleaned_data['body'],
        }

        if CONTACT_SEND_META_INFO:
            message_context['meta'] = self.get_request_meta()

        return render_to_string(self.message_template, message_context)


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['feedback_form', 'feedback_form.migrations']

package_data = \
{'': ['*'], 'feedback_form': ['templates/feedback_form/*']}

install_requires = \
['django>=3.0']

setup_kwargs = {
    'name': 'django-simple-feedback-form',
    'version': '2.0.1',
    'description': 'This is a Django application for providing a simple form for users to send emails to the administrators of your django-based site.',
    'long_description': '===========================\nDjango Simple Feedback Form\n===========================\n\nThis `Django <http://djangoproject.com>`_ app has for provide simple form for sending mail forom users to admins your django-based site.\n\nRequirements\n------------\n\nDjango 3.0+\n\n\nInstallation and configuration\n------------------------------\n\nSee `setup part`_ in documentation.\n\n.. _setup part: https://django-simple-feedback-form.readthedocs.io/en/latest/setup.html\n\nUsage\n-----\n\nSee `usage part`_ in documentation.\n\n.. _usage part: https://django-simple-feedback-form.readthedocs.io/en/latest/usage.html\n\n\nLicense\n-------\n\nLicensed under BSD license. See `license link`_ in documentation.\n\n.. _license link: LICENSE.rst',
    'author': 'Artem Galichkin',
    'author_email': 'doomer3d@gmail.com',
    'maintainer': 'Artem Galichkin',
    'maintainer_email': 'doomer3d@gmail.com',
    'url': 'https://github.com/DOOMer/django-simple-feedback-form',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6',
}


setup(**setup_kwargs)
